@@ -103,9 +103,7 @@ enum {
 	DEBUG_OPTION,
 	PREFIX_SEP_OPTION,
 	WARN_EMPTY_OPTION,
-#ifdef HAVE_UNAC
 	UNAC_OPTION,
-#endif
 };
 
 struct option long_options[] =
@@ -133,9 +131,7 @@ struct option long_options[] =
 	{"null", 0, 0, 'Z'},
 	{"match-prefix-separator", 1, 0, PREFIX_SEP_OPTION},
 	{"warn-empty", 0, 0, WARN_EMPTY_OPTION},
-#ifdef HAVE_UNAC
 	{"unac", 0, 0, UNAC_OPTION},
-#endif
 	{"fixed-strings", 0, 0, 'F'},
 	{0, 0, 0, 0}
 };
@@ -182,7 +178,7 @@ int search_in_document(poppler::document *doc, const std::string &filename, Rege
 	bool document_empty = true;
 
 	for (int i = 1; i <= doc->pages() && !max_count_reached; i++) {
-		std::auto_ptr<poppler::page> doc_page(doc->create_page(i - 1));
+		std::unique_ptr<poppler::page> doc_page(doc->create_page(i - 1));
 		if (!doc_page.get()) {
 			if (!quiet) {
 				fprintf(stderr, "pdfgrep: Could not search in page %d of %s\n", i, filename.c_str());
@@ -200,7 +196,7 @@ int search_in_document(poppler::document *doc, const std::string &filename, Rege
 
 		poppler::byte_array str = doc_page->text().to_utf8();
 		str.resize(str.size() + 1, '\0');
-		size_t str_len = str.size();
+		size_t str_len = str.size() - 1;
 #ifdef HAVE_UNAC
 		char *unac_str = simple_unac(&str[0]);
 		char *str_start = unac_str;
@@ -208,7 +204,7 @@ int search_in_document(poppler::document *doc, const std::string &filename, Rege
 		char *str_start = &str[0];
 #endif
 		size_t index = 0;
-		struct match mt = { .string = str_start, .strlen = str.size() };
+		struct match mt = { .string = str_start, .strlen = str_len };
 
 		while (!max_count_reached && !re.exec(str_start, index, &mt)) {
 			count_matches++;
@@ -253,10 +249,17 @@ int search_in_document(poppler::document *doc, const std::string &filename, Rege
 				}
 			}
 
-			index += mt.end;
+			found_something = 1;
+
+			index = mt.end;
+
+			// prevent loop if match is empty
+			if (mt.start == mt.end) {
+				index++;
+			}
+
 			if(index >= str_len)
 				break;
-			found_something = 1;
 		}
 
 #ifdef HAVE_UNAC
@@ -304,8 +307,8 @@ void parse_env_color_pair(char* pair, char** name, char** value)
 
 
 /* set colors of output according to content of environment-varaible env_var.
-    the content of env_var has to be like the GREP_COLORS variable for grep
-    see man 1 grep for further details of GREP_COLORS */
+   the content of env_var has to be like the GREP_COLORS variable for grep
+   see man 1 grep for further details of GREP_COLORS */
 void read_colors_from_env(const char* env_var)
 {
 	/* create a copy of var to edit it with strtok */
@@ -329,11 +332,13 @@ void read_colors_from_env(const char* env_var)
 		i += 1; /* skip delemiter */
 		/* get name and value of color pair */
 		parse_env_color_pair(cur_color_pair, &cur_name, &cur_value);
+
 #define PARSE_COLOR(GREP_NAME, GLOBAL_VAR) \
 	if (!strcmp(cur_name, GREP_NAME)) { \
-        free(GLOBAL_VAR); /* free old value */ \
+		free(GLOBAL_VAR); /* free old value */ \
 		GLOBAL_VAR = strdup(cur_value); /* set to new color */ \
-    }
+	}
+
 		/* now check for known settings and set global colors */
 		PARSE_COLOR("mt", outconf.colors.highlight)
 		else PARSE_COLOR("ms", outconf.colors.highlight)
@@ -341,7 +346,9 @@ void read_colors_from_env(const char* env_var)
 		else PARSE_COLOR("fn", outconf.colors.filename)
 		else PARSE_COLOR("ln", outconf.colors.pagenum)
 		else PARSE_COLOR("se", outconf.colors.separator)
+
 #undef PARSE_COLOR
+
 	}
 	/* free our copy of the environment var */
 	free(colors_list);
@@ -349,6 +356,10 @@ void read_colors_from_env(const char* env_var)
 
 void set_default_colors()
 {
+	// The grep(1) manpage documents the default value of GREP_COLORS to be
+	// ms=01;31:mc=01;31:sl=:cx=:fn=35:ln=32:bn=32:se=36
+	// which corresponds to these values below:
+
 	outconf.colors.filename = strdup("35");
 	outconf.colors.pagenum = strdup("32");
 	outconf.colors.highlight = strdup("01;31");
@@ -395,14 +406,14 @@ int get_line_width()
 void print_usage(char *self)
 {
 	printf("Usage: %s [OPTION]... PATTERN FILE...\n", self);
-	printf("See '%s --help' for more information\n", self);
+	printf("\nSee '%s --help' for more information\n", self);
 }
 
 void print_help(char *self)
 {
-	print_usage(self);
-	printf("\nSearch for PATTERN in each FILE.\n"
-"PATTERN is an extended regular expression.\n\n"
+	printf("Usage: %s [OPTION]... PATTERN FILE...\n"
+"\nSearch for PATTERN in each FILE.\n"
+"PATTERN is, by default, an extended regular expression.\n\n"
 
 "Options:\n"
 " -i, --ignore-case\t\tIgnore case distinctions\n"
@@ -420,7 +431,7 @@ void print_help(char *self)
 " -r, --recursive\t\tSearch directories recursively\n"
 " -R, --dereference-recursive\tLikewise, but follow all symlinks\n"
 "     --help\t\t\tPrint this help\n"
-" -V, --version\t\t\tShow version information\n");
+" -V, --version\t\t\tShow version information\n", self);
 }
 
 void print_version()
@@ -428,7 +439,10 @@ void print_version()
 	printf("This is %s version %s.\n", PACKAGE, VERSION);
 	printf("\nUsing poppler version %s\n", poppler::version_string().c_str());
 #ifdef HAVE_UNAC
-	printf("\nUsing unac version %s\n", unac_version());
+	printf("Using unac version %s\n", unac_version());
+#endif
+#ifdef HAVE_LIBPCRE
+	printf("Using libpcre version %s\n", pcre_version());
 #endif
 #ifdef PDFGREP_GIT_HEAD
 	printf("Built from git-commit %s\n", PDFGREP_GIT_HEAD);
@@ -482,7 +496,6 @@ int do_search_in_document(const std::string &path, const std::string &filename,
 int do_search_in_directory(const std::string &filename, Regengine &re)
 {
 	DIR *ptrDir = NULL;
-	struct dirent *ptrDirent = NULL;
 
 	ptrDir = opendir(filename.c_str());
 	if (!ptrDir) {
@@ -494,7 +507,7 @@ int do_search_in_directory(const std::string &filename, Regengine &re)
 	while(1) {
 		std::string path(filename);
 		errno = 0;
-		ptrDirent = readdir(ptrDir);    //not sorted, in order as `ls -f`
+		struct dirent *ptrDirent = ptrDirent = readdir(ptrDir);    //not sorted, in order as `ls -f`
 		if (!ptrDirent)
 			break;
 
@@ -601,14 +614,16 @@ int main(int argc, char** argv)
 				count = 1;
 				break;
 			case COLOR_OPTION:
-				if (!optarg)
-					break;
 				if (!strcmp("always", optarg)) {
 					outconf.color = 2;
 				} else if (!strcmp("never", optarg)) {
 					outconf.color = 0;
-				} else {
+				} else if (!strcmp("auto", optarg)) {
 					outconf.color = 1;
+				} else {
+					fprintf(stderr, "pdfgrep: Invalid argument '%s' for --color. "
+						"Candidates are: always, never or auto\n", optarg);
+					exit(EXIT_ERROR);
 				}
 				break;
 			case 'C':
@@ -665,11 +680,15 @@ int main(int argc, char** argv)
 			case DEBUG_OPTION:
 				debug = 1;
 				break;
-#ifdef HAVE_UNAC
+
 			case UNAC_OPTION:
+#ifndef HAVE_UNAC
+				fprintf(stderr, "pdfgrep: UNAC support disabled at compile time!\n");
+				exit(EXIT_ERROR);
+#else
 				use_unac = 1;
-				break;
 #endif
+				break;
 			case 'F':
 				re_engine |= RE_FIXED;
 				break;
@@ -771,9 +790,13 @@ int main(int argc, char** argv)
 		const std::string filename(argv[i]);
 
 		if (!is_dir(filename)) {
-			do_search_in_document(filename, filename, *re, false);
+			if (do_search_in_document(filename, filename, *re, false)) {
+				error = 1;
+			}
 		} else if (f_recursive_search) {
-			do_search_in_directory(filename, *re);
+			if (do_search_in_directory(filename, *re)) {
+				error = 1;
+			}
 		} else {
 			fprintf(stderr, "pdfgrep: %s is a directory\n", filename.c_str());
 			error = 1;
@@ -784,12 +807,15 @@ int main(int argc, char** argv)
 		do_search_in_directory(".", *re);
 	}
 
+	// Free up some stuff
+	delete re;
+
 	if (error) {
 		exit(EXIT_ERROR);
 	} else if (found_something) {
 		exit(EXIT_SUCCESS);
 	} else {
-		exit(EXIT_ERROR);
+		exit(EXIT_NOT_FOUND);
 	}
 }
 
